@@ -9,7 +9,7 @@
 #' @name SaemixModel-class
 #' @docType class
 #' @aliases SaemixModel-class SaemixModel [<-,SaemixModel-method
-#' @aliases plot,SaemixModel print,SaemixModel showall,SaemixModel show,SaemixModel summary,SaemixModel 
+#' @aliases print,SaemixModel showall,SaemixModel show,SaemixModel summary,SaemixModel 
 #' @section Objects from the Class: 
 #' An object of the SaemixModel class can be created by using the function \code{\link{saemixModel}} and contain the following slots:
 #'   \describe{
@@ -35,7 +35,9 @@
 #'     \item{showall}{\code{signature(object = "SaemixModel")}: shows all the elements in the object}
 #'     \item{show}{\code{signature(object = "SaemixModel")}: prints details about the object}
 #' 	 }
-#' @references Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear  mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
+#' @references Comets  E, Lavenu A, Lavielle M. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software 80, 3 (2017), 1-41.
+#' 
+#' Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
 #' 
 #' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm. 20th meeting of the 
 #' Population Approach Group in Europe, Athens, Greece (2011), Abstr 2173.
@@ -69,7 +71,7 @@ setClass(
     name.modpar="character",	# name of parameters in the model (columns of psi0)
     name.fixed="character",	# name of fixed parameters
     name.random="character",	# name of random parameters
-    name.res="character",	# name of residual parameters (maybe not necessary)
+    name.sigma="character",	# name of residual parameters (maybe not necessary)
     name.predictors="character",# name of predictors 
     name.X="character",	# name of X 
     name.response="character",	# name of response
@@ -83,7 +85,7 @@ setClass(
   validity=function(object){
 #    cat ("--- Checking SaemixModel object ---\n")
     if (dim(object@psi0)[1]==0) {
-      cat("[ SaemixModel : Error ] Please provide initial estimates for the fixed effect (a matrix with columns named after the parameters in the model).\n")
+      message("[ SaemixModel : Error ] Please provide initial estimates for the fixed effect (a matrix with columns named after the parameters in the model).")
       return("Missing psi0")
     }
     isize<-0
@@ -95,20 +97,20 @@ setClass(
     if (npar!=dim(object@omega.init)[1]) isize<-1
 #    cat("npar=",npar,length(object@transform.par),length(object@fixed.estim), dim(object@covariate.model)[2],dim(object@covariance.model)[1],dim(object@omega.init)[1],"\n")
     if(isize==1) {
-      cat("[ SaemixModel : Error ] The number of parameters should be the same in the following elements: psi0 (initial conditions), transform.par, fixed.estim, covariate.model, and the matrices covariance.model and omega.init should be square matrices of size equal to the number of parameters. Please check the input.\n")
+      message("[ SaemixModel : Error ] The number of parameters should be the same in the following elements: psi0 (initial conditions), transform.par, fixed.estim, covariate.model, and the matrices covariance.model and omega.init should be square matrices of size equal to the number of parameters. Please check the input.")
       return("Size mismatch")
     }
     if(npar<2) {
-      cat("[ SaemixModel : Error ] SAEM needs at least two parameters to work on.\n")
+      message("[ SaemixModel : Error ] SAEM needs at least two parameters to work on.")
       return("Psi0 has size 1")
     }
 		if(sum(object@fixed.estim*mydiag(object@covariance.model))==0) {
-			cat("[ SaemixModel : Error ] ")
-			if(sum(mydiag(object@covariance.model))==0) cat("At least one parameter with IIV must be included in the model.\n") else cat("At least one parameter with IIV must be estimated and not fixed in the model.\n")
+			message("[ SaemixModel : Error ] ")
+			if(sum(mydiag(object@covariance.model))==0) message("At least one parameter with IIV must be included in the model.") else message("At least one parameter with IIV must be estimated and not fixed in the model.")
 			return("Invalid IIV structure")
 		}
-    if(is.na(match(object@error.model,c('constant','proportional','combined', 'exponential')))) {
-      cat("[ SaemixModel : Error ] Invalid residual error model")
+    if(is.na(sum(match(object@error.model,c('constant','proportional','combined', 'exponential'))))) {
+      message("[ SaemixModel : Error ] Invalid residual error model")
       return("Invalid residual error model")
     }
     return(TRUE)
@@ -116,12 +118,49 @@ setClass(
 )
 
 #' @rdname initialize-methods
+#' 
+#' @param model name of the function used to compute the structural model. The
+#' function should return a vector of predicted values given a matrix of
+#' individual parameters, a vector of indices specifying which records belong
+#' to a given individual, and a matrix of dependent variables (see example
+#' below).
+#' @param description a character string, giving a brief description of the
+#' model or the analysis
+#' @param psi0 a matrix with a number of columns equal to the number of
+#' parameters in the model, and one (when no covariates are available) or two
+#' (when covariates enter the model) giving the initial estimates for the fixed
+#' effects. The column names of the matrix should be the names of the
+#' parameters in the model, and will be used in the plots and the summaries.
+#' When only the estimates of the mean parameters are given, psi0 may be a
+#' named vector.
+## #' @param name.response a character string or a column number specifying which column of the data contains the dependent variable
+#' @param name.sigma a vector of character string giving the names of the residual error parameters (defaults to "a" and "b")
+#' @param transform.par the distribution for each parameter (0=normal,
+#' 1=log-normal, 2=probit, 3=logit). Defaults to a vector of 1s (all parameters
+#' have a log-normal distribution)
+#' @param fixed.estim whether parameters should be estimated (1) or fixed to
+#' their initial estimate (0). Defaults to a vector of 1s
+#' @param error.model type of residual error model (valid types are constant,
+#' proportional, combined and exponential). Defaults to constant
+#' @param covariate.model a matrix giving the covariate model. Defaults to no
+#' covariate in the model
+#' @param covariance.model a square matrix of size equal to the number of parameters in the model, 
+#' giving the variance-covariance matrix of the model: 1s correspond to estimated variances (in the diagonal) 
+#' or covariances (off-diagonal elements). Defaults to the identity matrix
+#' @param omega.init a square matrix of size equal to the number of parameters
+#' in the model, giving the initial estimate for the variance-covariance matrix
+#' of the model. Defaults to the identity matrix
+#' @param error.init a vector of size 2 giving the initial value of a and b in
+#' the error model. Defaults to 1 for each estimated parameter in the error model
+#' @param name.modpar names of the model parameters, if they are not given as
+#' the column names (or names) of psi0
+#' 
 #' @exportMethod initialize
 
 setMethod(
   f="initialize",
   signature="SaemixModel",
-  definition=function(.Object,model,description,psi0,transform.par,fixed.estim, error.model,covariate.model,covariance.model,omega.init,error.init,nb.parameters, name.modpar, verbose=TRUE){
+  definition=function(.Object,model,description,psi0, name.response, name.sigma, transform.par,fixed.estim, error.model,covariate.model,covariance.model,omega.init,error.init, name.modpar, verbose=TRUE){
 #    cat ("--- initialising SaemixModel Object --- \n")
     if(missing(model)) {
 #      cat("Error initialising SaemixModel object:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
@@ -131,24 +170,26 @@ setMethod(
     if(missing(description)) description<-""
     .Object@description<-description
     if(missing(psi0) || length(psi0)==0) {
-      cat("Error initialising SaemixModel object:\n   Please provide initial estimates for the fixed effect (a matrix with columns named after the parameters in the model).\n")
+      if(verbose) message("Error initialising SaemixModel object:\n   Please provide initial estimates for the fixed effect (a matrix with columns named after the parameters in the model).\n")
       return(.Object)
     }
     npar<-dim(psi0)[2]
     if(missing(name.modpar) || length(name.modpar)==0) {
       y1<-try(name.modpar<-colnames(psi0))
       if(class(y1)=="try-error") {
-        if(verbose) cat("     Can't find parameter names.\n")
+        if(verbose) message("     Can't find parameter names.\n")
         name.modpar<-paste("theta",1:npar)
       }
     }
     if(is.null(colnames(psi0))) {
       y1<-try(colnames(psi0)<-name.modpar)
       if(class(y1)=="try-error") {
-        if(verbose) cat("Warning:\n   Problem with names of psi0\n")
+        if(verbose) message("Warning:\n   Problem with names of psi0\n")
         colnames(psi0)<-name.modpar<-paste("theta",1:npar)
       }
     }
+    if(missing(name.response)) name.response<-""
+    .Object@name.response<-name.response
     if(!missing(covariate.model)) {
     	if(dim(psi0)[1]<2 & sum(covariate.model)>0){
     		psi0<-rbind(psi0,rep(0,dim(psi0)[2]))
@@ -162,6 +203,11 @@ setMethod(
     .Object@psi0<-psi0    
     .Object@name.modpar<-name.modpar
     if(missing(error.model) || length(error.model)==0) error.model<-"constant"
+    if(sum(!error.model %in% c('constant','proportional','combined', 'exponential'))) {
+      message("Invalid error model, switching to constant")
+      error.model[!error.model %in% c('constant','proportional','combined', 'exponential')] <- "constant"
+    }
+    if(length(error.model)<length(name.response)) error.model<-rep(error.model,length.out=length(name.response))
     .Object@error.model<-error.model
 # Checking sizes
     .Object@nb.parameters<-npar
@@ -170,19 +216,20 @@ setMethod(
     if(missing(fixed.estim) || length(fixed.estim)==0) fixed.estim<-rep(1,npar)
     .Object@fixed.estim<-fixed.estim
     if(missing(covariate.model) || length(covariate.model)==0 || sum(covariate.model)==0) covariate.model<-matrix(nrow=0,ncol=npar)
+    if(is.null(dim(covariate.model)) & length(covariate.model)>0) covariate.model<-matrix(covariate.model,byrow=T,ncol=npar) # Covariate model given as a vector
     if(is.null(colnames(covariate.model))) colnames(covariate.model)<-colnames(psi0)
     .Object@covariate.model<-covariate.model
     if(missing(covariance.model) || length(covariance.model)==0) {
       covariance.model<-diag(nrow=npar,ncol=npar)
     } else {
       if(dim(covariance.model)[1]!=dim(covariance.model)[2]) {
-        cat("Error initialising SaemixModel object:\n   The covariance model needs to be a square matrix, please check dimensions.\n")
+        if(verbose) message("Error initialising SaemixModel object:\n   The covariance model needs to be a square matrix, please check dimensions.\n")
       return(.Object)
       }
     }
     nomg<-dim(covariance.model)[1]
     if(nomg!=npar) {
-      cat("Error initialising SaemixModel object:\n   The covariance model needs to have the same size as the number of parameters.\n")
+      if(verbose) message("Error initialising SaemixModel object:\n   The covariance model needs to have the same size as the number of parameters.\n")
       return(.Object)
     }
     if(is.null(colnames(covariance.model))) colnames(covariance.model)<-rownames(covariance.model)<-colnames(psi0)
@@ -191,7 +238,7 @@ setMethod(
     .Object@indx.omega<-indx.omega
     if(!missing(omega.init) && length(omega.init)>0) {
       if(dim(omega.init)[1]!=dim(omega.init)[2]) {
-        if(verbose) cat("Warning:   the matrix giving the initial conditions for the covariance model (omega.init) needs to be a square matrix. Changing it to the diagonal matrix\n")
+        if(verbose) message("Warning:   the matrix giving the initial conditions for the covariance model (omega.init) needs to be a square matrix. Changing it to the diagonal matrix\n")
         omega.init<-NULL
       }
     }
@@ -205,10 +252,12 @@ setMethod(
       }
       omega.init<-diag(diag.omegi,nrow=npar)
     }
-    if(is.null(colnames(omega.init))) colnames(omega.init)<-rownames(omega.init)<-colnames(psi0)
+    if(is.null(colnames(omega.init))) {
+      if(dim(omega.init)[1]==length(colnames(psi0))) colnames(omega.init)<-rownames(omega.init)<-colnames(psi0) else message("The dimensions of omega.init don't agree with the number of parameters")
+    }
     .Object@omega.init<-omega.init
 		if(sum(.Object@fixed.estim*mydiag(.Object@covariance.model))==0) {
-			cat("Error initialising SaemixModel object:\n")
+			message("Error initialising SaemixModel object:\n")
 #			if(sum(mydiag(.Object@covariance.model))==0) cat("At least one parameter with IIV must be included in the model.\n") else cat("At least one parameter with IIV must be estimated and not fixed in the model.\n")
 			return(.Object)
 		}
@@ -220,31 +269,44 @@ setMethod(
 #   proportional        y = f + b*f*e
 #   combined            y = f + (a+b*f)*e
 #   exponential         y = f*exp(a*e)    ( <=>  log(y) = log(f) + a*e )
-    if(missing(error.init) || length(error.init)!=2) {
-      error.init<-switch(error.model,
+    if(missing(error.init) || length(error.init)!=2*length(.Object@error.model)) {
+      error.init<-c()
+      for(i in 1:length(.Object@error.model)) {
+        error.init<-c(error.init,switch(error.model[i],
         "constant"=c(1,0),
         "exponential"=c(1,0),
         "proportional"=c(0,1),
-        "combined"=c(1,1))
+            "combined"=c(1,1)))
      }
+    }
+    xres<-c()
+    if(missing(name.sigma)) mis.sig<-TRUE else mis.sig<-FALSE
+    if(missing(name.sigma) || length(name.sigma)!=2) name.sigma<-c("a","b")
+    if(!mis.sig) { # & .Object@name.response!="" # pb if response has more than 1 element
+      for(i in 1:length(.Object@name.response)) xres<-c(xres,paste(name.sigma,.Object@name.response[i],sep="."))
+    } else xres<-rep(name.sigma,length(.Object@name.response))
+    .Object@name.sigma<-xres
     .Object@error.init<-error.init
-    .Object@name.res<-c("a","b")
-    if(.Object@error.model=='constant') {
-      indx.res<-1
+    indx.res<-c()
+    for(i in 1:length(.Object@error.model)) {
+      if(.Object@error.model[i]=='constant') {
+        indx.res1<-1
     } else {
-      if(.Object@error.model=='proportional') {
-        indx.res<-2
+        if(.Object@error.model[i]=='proportional') {
+          indx.res1<-2
       } else {
-        if(.Object@error.model=='combined') {
-          indx.res<-c(1,2) 
+          if(.Object@error.model[i]=='combined') {
+            indx.res1<-c(1,2) 
         } else {
-          if(.Object@error.model=='exponential') {
-           indx.res<-1
+            if(.Object@error.model[i]=='exponential') {
+              indx.res1<-1
            }
         }
       }
     }
-    if(length(indx.res<2)) .Object@error.init[-indx.res]<-0
+      indx.res<-c(indx.res,indx.res1+2*(i-1))
+    }
+    .Object@error.init[-indx.res]<-0
     .Object@indx.res<-indx.res
     .Object@betaest.model<-matrix(c(rep(1,.Object@nb.parameters), c(t(.Object@covariate.model))),ncol=.Object@nb.parameters,byrow=TRUE)
     colnames(.Object@betaest.model)<-colnames(.Object@covariate.model)
@@ -298,7 +360,7 @@ setMethod(
     "name.modpar"={return(x@name.modpar)},
     "name.fixed"={return(x@name.fixed)},
     "name.random"={return(x@name.random)},
-    "name.res"={return(x@name.res)},
+    "name.sigma"={return(x@name.sigma)},
     "name.X"={return(x@name.X)},
     "name.response"={return(x@name.response)},
     "name.predictors"={return(x@name.predictors)},
@@ -334,7 +396,7 @@ setReplaceMethod(
     "name.modpar"={x@name.modpar<-value},
     "name.fixed"={x@name.fixed<-value},
     "name.random"={x@name.random<-value},
-    "name.res"={x@name.res<-value},
+    "name.sigma"={x@name.sigma<-value},
     "name.X"={x@name.X<-value},
     "name.response"={x@name.response<-value},
     "name.predictors"={x@name.predictors<-value},
@@ -375,7 +437,7 @@ setMethod("print","SaemixModel",
     tab<-x@covariance.model
 #    try(colnames(tab)<-rownames(tab)<-x@name.modpar)
     print(tab,quote=FALSE)
-    st1<-paste(x@name.res,x@error.init,sep="=")
+    st1<-paste(x@name.sigma,x@error.init,sep="=")
     cat("  Error model:",x@error.model,", initial values:",st1[x@indx.res],"\n")
    if(dim(x@covariate.model)[1]>0) {
       cat("  Covariate model:")
@@ -427,7 +489,7 @@ setMethod("showall","SaemixModel",
     cat("      parameter names: ",object@name.modpar,"\n")
     if(length(object@name.fixed)>0) cat("      fixed parameters: ",object@name.fixed,"\n")
     if(length(object@name.random)>0) cat("      random parameters: ",object@name.random,"\n")
-    if(length(object@name.res)>0) cat("      parameters of residual variability: ",object@name.res,"\n")
+    if(length(object@name.sigma)>0) cat("      parameters of residual variability: ",object@name.sigma,"\n")
     if(length(object@name.predictors)>0) cat("      predictors: ",object@name.predictors,"\n")
     if(length(object@name.X)>0) cat("      X predictor: ",object@name.X,"\n")
     if(length(object@name.cov)>0) cat("      covariates: ",object@name.cov,"\n")
@@ -439,7 +501,7 @@ setMethod("showall","SaemixModel",
     print(tab,quote=FALSE)
     cat("  Initial estimate for variance-covariance matrix:\n")
     print(object@omega.init)
-    st1<-paste(object@name.res,object@error.init,sep="=")
+    st1<-paste(object@name.sigma,object@error.init,sep="=")
     cat("  Error model:",object@error.model,", initial values:",st1[object@indx.res],"\n")
    if(dim(object@covariate.model)[1]>0) {
       cat("  Covariate model:")
@@ -487,7 +549,7 @@ setMethod("summary","SaemixModel",
     }
     distrib<-c("normal","log-normal","probit","logit")
     tab.par<-data.frame(Parameter=object@name.modpar, Distribution=distrib[object@transform.par+1], Estimated=ifelse(as.numeric(object@betaest.model[1,])==1,"estimated","fixed"), Initial.value=object@psi0[1,])
-    tab.res<-data.frame(parameters=object@name.res,Initial.value=object@error.init)   
+    tab.res<-data.frame(parameters=object@name.sigma,Initial.value=object@error.init)   
     res<-list(model=list(model.function=object@model, error.model=object@error.model),parameters=list(fixed=tab.par, residual.error=tab.res),covariance.model=object@covariance.model, covariate.model=object@covariate.model)
     invisible(res)
  }
@@ -497,12 +559,30 @@ setMethod("summary","SaemixModel",
 ####			SaemixModel class - method to plot			####
 ####################################################################################
 
-#' @rdname plotmodel-methods
-#' @exportMethod plotmodel
+#' Plot model predictions using an SaemixModel object
+#' 
+#' This function will plot predictions obtained from an SaemixModel object over a given range of X. Additional predictors may be passed on to the function using the predictors argument.
+#' 
+#' @name plot-SaemixModel
+#' 
+#' @param x an SaemixData object or an SaemixSimData object
+#' @param y unused, present for compatibility with base plot function
+#' @param range range of X over which the model is to be plotted
+#' @param psi parameters of the model 
+#' @param predictors additional predictors needed to pass on to the model
+#' @param ... additional arguments to be passed on to plot (titles, legends, ...)
+#' 
+#' @aliases plot,SaemixModel-methods 
+#' @aliases plot,SaemixModel
+#' @keywords methods
+### #' @docType methods
+#' @exportMethod plot
+#' @rdname plot-SaemixModel
 
-# Plot predictions from the model, using a set of parameters (defaulting to the population parameters stored in the object)
+# Plot simulations from the model
+# ECO TODO: adjust to multiple responses
 
-setMethod("plotmodel","SaemixModel",
+setMethod("plot","SaemixModel",
   function(x,y,range=c(0,1),psi,predictors,...) {
     if(missing(psi)) psi<-x@psi0[1,]
     psi<-matrix(psi,nrow=1)
@@ -517,6 +597,10 @@ setMethod("plotmodel","SaemixModel",
       }
      }
     }
+    if(length(x@name.response)>1) {
+      cat("Currently the plot can only be obtained for single-response models.\n")
+      return()
+    }
     npts<-100
     psi<-matrix(rep(psi,npts+1),byrow=T,nrow=(npts+1))
     id<-matrix(rep(1,npts+1),ncol=1)
@@ -586,6 +670,8 @@ setMethod("plotmodel","SaemixModel",
 #' named vector.
 #' @param description a character string, giving a brief description of the
 #' model or the analysis
+#' @param name.response the name of the dependent variable
+#' @param name.sigma a vector of character string giving the names of the residual error parameters
 #' @param error.model type of residual error model (valid types are constant,
 #' proportional, combined and exponential). Defaults to constant
 #' @param transform.par the distribution for each parameter (0=normal,
@@ -613,9 +699,9 @@ setMethod("plotmodel","SaemixModel",
 #' Marc Lavielle.
 #' @seealso \code{\link{SaemixData}},\code{\link{SaemixModel}},
 #' \code{\link{saemixControl}},\code{\link{saemix}}
-#' @references Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear
-#' mixed effects models. Computational Statistics and Data Analysis 49, 4
-#' (2005), 1020-1038.
+#' @references Comets  E, Lavenu A, Lavielle M. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software 80, 3 (2017), 1-41.
+#' 
+#' Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
 #' 
 #' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm.
 #' 20th meeting of the Population Approach Group in Europe, Athens, Greece
@@ -644,33 +730,33 @@ setMethod("plotmodel","SaemixModel",
 #' 
 #' @export saemixModel
 
-saemixModel<-function(model,psi0,description="",error.model=character(), transform.par=numeric(),fixed.estim=numeric(),covariate.model=matrix(nrow=0,ncol=0), covariance.model=matrix(nrow=0,ncol=0),omega.init=matrix(nrow=0,ncol=0),error.init=numeric(), name.modpar=character(), verbose=TRUE) {
+saemixModel<-function(model,psi0,description="", name.response="", name.sigma=character(), error.model=character(), transform.par=numeric(),fixed.estim=numeric(),covariate.model=matrix(nrow=0,ncol=0), covariance.model=matrix(nrow=0,ncol=0),omega.init=matrix(nrow=0,ncol=0),error.init=numeric(), name.modpar=character(), verbose=TRUE) {
 # Creating model from class
   if(missing(model)) {
-    cat("Error in saemixModel:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
+    if(verbose) cat("Error in saemixModel:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
     return("Creation of SaemixModel failed")  
   }
   xcal<-try(typeof(model))
   if(class(xcal)=="try-error") {
-    cat("Error in saemixModel:\n   the model function does not exist.\n")
+    if(verbose) cat("Error in saemixModel:\n   the model function does not exist.\n")
     return("Creation of SaemixModel failed")  
   }
   if(typeof(model)=="character") {
     if(exists(model)) model<-get(model) else {
-      cat("Error in saemixModel:\n   The argument model to saemixModel must be a valid function.\n")
+      if(verbose) cat("Error in saemixModel:\n   The argument model to saemixModel must be a valid function.\n")
       return("Creation of SaemixModel failed")
     }
   }
   if(!is.function(model)) {
-    cat("Error in saemixModel:\n   The argument model to saemixModel must be a valid function.\n")
+    if(verbose) cat("Error in saemixModel:\n   The argument model to saemixModel must be a valid function.\n")
     return("Creation of SaemixModel failed")
   }
   if(length(formals(model))!=3) {
-    cat("Error in saemixModel:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
+    if(verbose) cat("Error in saemixModel:\n   The model must be a function, accepting 3 arguments: psi (a vector of parameters), id (a vector of indices) and xidep (a matrix of predictors). Please see the documentation for examples.\n")
     return("Creation of SaemixModel failed")
   }
   if(missing(psi0) || length(psi0)==0) {
-    cat("Error in saemixModel:\n   please provide initial estimates psi0 for at least the fixed effects.\n")
+    if(verbose) cat("Error in saemixModel:\n   please provide initial estimates psi0 for at least the fixed effects.\n")
     return("Creation of SaemixModel failed")  
   }
   if(is.null(dim(psi0))) {
@@ -682,7 +768,7 @@ saemixModel<-function(model,psi0,description="",error.model=character(), transfo
   if(is.null(colnames(psi0))) {
     if(verbose) cat("Warning: no names given for the parameters in the model, please consider including parameter names.\n")
   }
-  xmod<-try(new(Class="SaemixModel",model=model,description=description,psi0=psi0, error.model=error.model, transform.par=transform.par,fixed.estim=fixed.estim, covariate.model=covariate.model,covariance.model=covariance.model, omega.init=omega.init,error.init=error.init,name.modpar=name.modpar))
+  xmod<-try(new(Class="SaemixModel",model=model,description=description ,psi0=psi0, name.response=name.response, name.sigma=name.sigma, error.model=error.model, transform.par=transform.par,fixed.estim=fixed.estim, covariate.model=covariate.model,covariance.model=covariance.model, omega.init=omega.init,error.init=error.init,name.modpar=name.modpar))
   if(class(xmod)=="SaemixModel") x1<-try(validObject(xmod),silent=FALSE) else x1<-xmod
   if(class(x1)!="try-error") {
     if(verbose) cat("\n\nThe following SaemixModel object was successfully created:\n\n")
@@ -691,4 +777,57 @@ saemixModel<-function(model,psi0,description="",error.model=character(), transfo
   return(xmod)
 }
 
+####################################################################################
+####  Auxiliary function
+
+#' Matrix diagonal
+#' 
+#' Extract or replace the diagonal of a matrix, or construct a diagonal matrix (replace diag function from R-base)
+#' 
+#' @param x	a matrix, vector or 1D array, or missing.
+#' @param nrow, ncol Optional dimensions for the result when x is not a matrix. 
+#' @param value either a single value or a vector of length equal to that of the current diagonal. Should be of a mode which can be coerced to that of x.
+#' @return If x is a matrix then diag(x) returns the diagonal of x. The resulting vector will have names if the matrix x has matching column and rownames.
+#' \seealso{\code{diag}}
+#' @author Emmanuelle Comets <emmanuelle.comets@@inserm.fr>, Audrey Lavenu,
+#' Marc Lavielle.
+#' @keywords models
+#' @examples
+#' 
+#' mydiag(1)
+#' mydiag(c(1,2))
+#' 
+#' @export mydiag
+
+# Redefining diag function, too many problems with the R version
+mydiag <- function (x = 1, nrow, ncol) {
+  if (is.matrix(x)) {
+    if (nargs() > 1L) 
+      stop("'nrow' or 'ncol' cannot be specified when 'x' is a matrix")
+    if ((m <- min(dim(x))) == 0L) 
+      return(vector(typeof(x), 0L))
+    y <- c(x)[1L + 0L:(m - 1L) * (dim(x)[1L] + 1L)]
+    nms <- dimnames(x)
+    if (is.list(nms) && !any(sapply(nms, is.null)) && identical((nm <- nms[[1L]][seq_len(m)]), 
+                                                                nms[[2L]][seq_len(m)])) 
+      names(y) <- nm
+    return(y)
+  }
+  if (is.array(x) && length(dim(x)) != 1L) 
+    stop("'x' is an array, but not 1D.")
+  if (missing(x)) 
+    n <- nrow
+  else n <- length(x)
+  if (!missing(nrow)) 
+    n <- nrow
+  if (missing(ncol)) 
+    ncol <- n
+  p <- ncol
+  y <- array(0, c(n, p))
+  if ((m <- min(n, p)) > 0L) 
+    y[1L + 0L:(m - 1L) * (n + 1L)] <- x
+  y
+}
+
+
 ####################################################################################
