@@ -61,9 +61,9 @@
 #' Marc Lavielle.
 #' @seealso \code{\link{SaemixData}},\code{\link{SaemixModel}},
 #' \code{\link{SaemixObject}},\code{\link{saemixControl}},\code{\link{saemix}}
-#' @references Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear
-#' mixed effects models. Computational Statistics and Data Analysis 49, 4
-#' (2005), 1020-1038.
+#' @references Comets  E, Lavenu A, Lavielle M. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software 80, 3 (2017), 1-41.
+#' 
+#' Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
 #' 
 #' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm.
 #' 20th meeting of the Population Approach Group in Europe, Athens, Greece
@@ -118,7 +118,9 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
   # nsamp= number of MCMC samples
   # kmax= max nb of iterations
   # returns an array 
-  N<-saemixObject["data"]["N"]
+  saemix.data<-saemixObject["data"]
+  saemix.model<-saemixObject["model"]
+  N<-saemix.data["N"]
   nb.parameters<-saemixObject["model"]["nb.parameters"]
   if(is.null(max.iter)) kmax<-sum(saemixObject["options"]$nbiter.saemix)*2 else kmax<-max.iter
   # using several Markov chains
@@ -126,7 +128,7 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
   NM<-chdat["NM"]
   IdM<-chdat["dataM"]$IdM
   yM<-chdat["dataM"]$yM
-  XM<-chdat["dataM"][,saemixObject["data"]["name.predictors"],drop=FALSE]
+  XM<-chdat["dataM"][,c(saemix.data["name.predictors"],saemix.data["name.cens"],saemix.data["name.mdv"],saemix.data["name.ytype"]),drop=FALSE]
   io<-matrix(data=0,nrow=N,ncol=max(saemixObject["data"]["nind.obs"]))
   for(i in 1:N)
     io[i,1:saemixObject["data"]["nind.obs"][i]]<-1
@@ -176,9 +178,9 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
   etaM<-phiM[,ind.eta]-mean.phiM[,ind.eta]  
   psiM<-transphi(phiM,saemixObject["model"]["transform.par"])
   fpred<-saemixObject["model"]["model"](psiM, IdM, XM)
-  if(saemixObject["model"]["error.model"]=="exponential")
-    fpred<-log(cutoff(fpred))
-  gpred<-error(fpred,pres)
+  ind.exp<-which(saemix.model["error.model"]=="exponential")
+  for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
+  gpred<-error(fpred,pres,XM$ytype)
   DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)^2+log(gpred)
   U.y<-colSums(DYF)
   phiMc<-phiM
@@ -194,9 +196,8 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
       phiMc[,ind.eta]<-mean.phiM[,ind.eta]+etaMc
       psiMc<-transphi(phiMc,saemixObject["model"]["transform.par"])
       fpred<-saemixObject["model"]["model"](psiMc, IdM, XM)
-      if(saemixObject["model"]["error.model"]=="exponential")
-        fpred<-log(cutoff(fpred))
-      gpred<-error(fpred,pres)
+      for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
+      gpred<-error(fpred,pres,XM$ytype)
       DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)^2+log(gpred)
       Uc.y<-colSums(DYF)
       deltau<-Uc.y-U.y
@@ -219,9 +220,8 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
           phiMc[,ind.eta]<-mean.phiM[,ind.eta]+etaMc
           psiMc<-transphi(phiMc,saemixObject["model"]["transform.par"])
           fpred<-saemixObject["model"]["model"](psiMc, IdM, XM)
-          if(saemixObject["model"]["error.model"]=="exponential")
-            fpred<-log(cutoff(fpred))
-          gpred<-error(fpred,pres)
+          for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
+          gpred<-error(fpred,pres,XM$ytype)
           DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)**2+log(gpred)
           Uc.y<-colSums(DYF) # Warning: Uc.y, Uc.eta = vecteurs
           Uc.eta<-0.5*rowSums(etaMc*(etaMc%*%solve(omega.eta)))
@@ -256,9 +256,8 @@ conddist.saemix<-function(saemixObject,nsamp=1,max.iter=NULL,...) {
           phiMc[,ind.eta]<-mean.phiM[,ind.eta]+etaMc
           psiMc<-transphi(phiMc,saemixObject["model"]["transform.par"])
           fpred<-saemixObject["model"]["model"](psiMc, IdM, XM)
-          if(saemixObject["model"]["error.model"]=="exponential")
-            fpred<-log(cutoff(fpred))
-          gpred<-error(fpred,pres)
+          for(ityp in ind.exp) fpred[XM$ytype==ityp]<-log(cutoff(fpred[XM$ytype==ityp]))
+          gpred<-error(fpred,pres,XM$ytype)
           DYF[ind.ioM]<-0.5*((yM-fpred)/gpred)**2+log(gpred)
           Uc.y<-colSums(DYF) # Warning: Uc.y, Uc.eta = vecteurs
           Uc.eta<-0.5*rowSums(etaMc*(etaMc%*%solve(omega.eta)))
