@@ -30,7 +30,6 @@ initialiseMainAlgo<-function(saemix.data,saemix.model,saemix.options) {
 #             alpha0.sa=10^(-3/saemix.options$nbiter.sa),nbiter.saemix=saemix.options$nbiter.saemix,
 #             maxim.maxiter=saemix.options$maxim.maxiter,flag.fmin=flag.fmin)
   
-  
 	structural.model<-saemix.model["model"]
 	nb.parameters<-saemix.model["nb.parameters"]
 	N<-saemix.data["N"]
@@ -42,7 +41,7 @@ initialiseMainAlgo<-function(saemix.data,saemix.model,saemix.options) {
 	#   combined            y = f + (a+b*f)*e
 	#   exponential         y = f*exp(a*e)    ( <=>  log(y) = log(f) + a*e )
 	# error models are a + bf described by [a b], [1]=constant coefficient, [2]= proportional coefficient
-	pres<-c(saemix.model["error.init"][1],saemix.model["error.init"][2])
+	pres<-saemix.model["error.init"]
 	
 	# ECO TODO: integrate all this section in the object creation ?
 	# Initialisation: 
@@ -166,7 +165,7 @@ initialiseMainAlgo<-function(saemix.data,saemix.model,saemix.options) {
 	NM<-chdat["NM"]
 	IdM<-chdat["dataM"]$IdM
 	yM<-chdat["dataM"]$yM
-	XM<-chdat["dataM"][,saemix.data["name.predictors"],drop=FALSE]
+	XM<-chdat["dataM"][,c(saemix.data["name.predictors"],saemix.data["name.cens"],saemix.data["name.mdv"],saemix.data["name.ytype"]),drop=FALSE]
 	io<-matrix(data=0,nrow=N,ncol=max(saemix.data["nind.obs"]))
 	for(i in 1:N)
 		io[i,1:saemix.data["nind.obs"][i]]<-1
@@ -232,7 +231,7 @@ initialiseMainAlgo<-function(saemix.data,saemix.model,saemix.options) {
 	# Data - passed on to functions, unchanged
 	Dargs<-list(IdM=IdM, XM=XM, yM=yM, NM=NM, N=N, nobs=saemix.data["ntot.obs"],
 							yobs=saemix.data["data"][,saemix.data["name.response"]],transform.par=saemix.model["transform.par"],
-							error.model=saemix.model["error.model"],structural.model=structural.model)
+							error.model=saemix.model["error.model"],structural.model=structural.model , etype.exp=which(saemix.model["error.model"] == "exponential"))
 	
 	# List of indices and variables (fixed) - passed on to functions, unchanged
 	nb.parest<-sum(covariate.estim)+ sum(saemix.model["covariance.model"][upper.tri(saemix.model["covariance.model"], diag=TRUE)])+1+as.integer(saemix.model["error.model"]=="combined")
@@ -246,7 +245,8 @@ initialiseMainAlgo<-function(saemix.data,saemix.model,saemix.options) {
 	# Variability-related elements
 	omega.eta<-omega[ind.eta,ind.eta] # IIV matrix for estimated parameters
 	varList<-list(pres=pres,ind0.eta=ind0.eta,ind.eta=ind.eta,omega=omega, MCOV=MCOV,
-								domega2=do.call(cbind,rep(list((sqrt(mydiag(omega.eta)))*saemix.options$rw.ini),nb.etas)),diag.omega=mydiag(omega))
+								domega2=do.call(cbind,rep(list((sqrt(mydiag(omega.eta)))*saemix.options$rw.ini),nb.etas)),
+								diag.omega=mydiag(omega))
 	# List of options and settings (fixed) - passed on to functions, unchanged
 	stepsize<-rep(1,saemix.options$nbiter.tot)
 	stepsize[(saemix.options$nbiter.saemix[1]+1):saemix.options$nbiter.tot]<-1/
